<?php

/**
 * @file
 * Provides admin menu callbacks for Commerce Square.
 */
use SquareConnect\ApiException;

/**
 * Form callback: allows the user to capture a prior authorization.
 */
function commerce_square_capture_form($form, &$form_state, $order, $transaction) {
  $form_state['order'] = $order;
  $form_state['transaction'] = $transaction;

  // Load and store the payment method instance for this transaction.
  $payment_method = commerce_payment_method_instance_load($transaction->instance_id);
  $form_state['payment_method'] = $payment_method;

  $form = confirm_form($form,
    t('Are you sure you want to capture this transaction?'),
    'admin/commerce/orders/' . $order->order_id . '/payment',
    '',
    t('Capture'),
    t('Cancel'),
    'confirm'
  );

  return $form;
}

/**
 * Validate handler: check the credit amount before attempting credit request.
 */
function commerce_square_capture_form_validate($form, &$form_state) {
  $transaction = $form_state['transaction'];
  if (time() - $transaction->created > 86400 * 6) {
    drupal_set_message(t('This capture has passed its 6 day limit for issuing a capture.'), 'error');
    drupal_goto('admin/commerce/orders/' . $form_state['order']->order_id . '/payment');
  }
}

/**
 * Submit handler: process the void request.
 */
function commerce_square_capture_form_submit($form, &$form_state) {
  $transaction = $form_state['transaction'];
  $client = SquareApi::createFromInstanceId($transaction->instance_id);

  try {
    $client->capture($transaction);
    $transaction->status = COMMERCE_PAYMENT_STATUS_SUCCESS;
    drupal_set_message(t('Transaction successfully captured.'));
  }
  catch (ApiException $e) {
    /** @var \SquareConnect\Model\VoidTransactionResponse $result */
    $result = $e->getResponseObject();
    foreach ($result->getErrors() as $error) {
      drupal_set_message(sprintf('%s %s %s', $error->getCode(), $error->getCategory(), $error->getDetail()), 'error');
    }
  }
  catch (Exception $e) {
    drupal_set_message(t('There was an error capturing the transaction: @message', array('@message' => $e->getMessage())), 'error');
  }

  commerce_payment_transaction_save($transaction);

  $form_state['redirect'] = 'admin/commerce/orders/' . $form_state['order']->order_id . '/payment';
}

/**
 * Form callback: allows the user to void a transaction.
 */
function commerce_square_void_form($form, &$form_state, $order, $transaction) {
  $form_state['order'] = $order;
  $form_state['transaction'] = $transaction;

  // Load and store the payment method instance for this transaction.
  $payment_method = commerce_payment_method_instance_load($transaction->instance_id);
  $form_state['payment_method'] = $payment_method;

  $form['markup'] = array(
    '#markup' => t('Are you sure that you want to void this transaction?'),
  );

  $form = confirm_form($form,
    t('Are you sure that you want to void this transaction?'),
    'admin/commerce/orders/' . $order->order_id . '/payment',
    '',
    t('Void'),
    t('Cancel'),
    'confirm'
  );

  return $form;
}

/**
 * Submit handler: process the void request.
 */
function commerce_square_void_form_submit($form, &$form_state) {
  $transaction = $form_state['transaction'];
  $client = SquareApi::createFromInstanceId($transaction->instance_id);

  try {
    $client->void($transaction);
    $transaction->status = COMMERCE_PAYMENT_STATUS_FAILURE;
    drupal_set_message(t('Transaction successfully voided.'));
  }
  catch (ApiException $e) {
    /** @var \SquareConnect\Model\VoidTransactionResponse $result */
    $result = $e->getResponseObject();
    foreach ($result->getErrors() as $error) {
      drupal_set_message(sprintf('%s %s %s', $error->getCode(), $error->getCategory(), $error->getDetail()), 'error');
    }
  }
  catch (Exception $e) {
    drupal_set_message(t('There was an error voiding the transaction: @message', array('@message' => $e->getMessage())), 'error');
  }

  commerce_payment_transaction_save($transaction);

  $form_state['redirect'] = 'admin/commerce/orders/' . $form_state['order']->order_id . '/payment';
}

/**
 * Form callback: allows the user to refund a transaction.
 */
function commerce_square_refund_form($form, &$form_state, $order, $transaction) {
  $form_state['order'] = $order;
  $form_state['transaction'] = $transaction;

  // Load and store the payment method instance for this transaction.
  $payment_method = commerce_payment_method_instance_load($transaction->instance_id);
  $form_state['payment_method'] = $payment_method;

  $default_amount = number_format(commerce_currency_amount_to_decimal($transaction->amount, $transaction->currency_code), 2, '.', '');

  $form['amount'] = array(
    '#type' => 'textfield',
    '#title' => t('Refund amount'),
    '#description' => t('Enter the amount to be refunded.'),
    '#default_value' => $default_amount,
    '#field_suffix' => check_plain($transaction->currency_code),
    '#size' => 16,
  );

  $form = confirm_form($form,
    t('What amount do you want to refund?'),
    'admin/commerce/orders/' . $order->order_id . '/payment',
    '',
    t('Refund'),
    t('Cancel'),
    'confirm'
  );

  return $form;
}

/**
 * Validate handler: check the credit amount before attempting credit request.
 */
function commerce_square_refund_form_validate($form, &$form_state) {
  $transaction = $form_state['transaction'];
  $amount = $form_state['values']['amount'];

  // Ensure a positive numeric amount has been entered for credit.
  if (!is_numeric($amount) || $amount <= 0) {
    form_set_error('amount', t('You must specify a positive numeric amount to refund.'));
  }

  // Ensure the amount is less than or equal to the captured amount.
  if ($amount > commerce_currency_amount_to_decimal($transaction->amount, $transaction->currency_code)) {
    form_set_error('amount', t('You cannot credit more than you captured through Square.'));
  }

  if (time() - $transaction->created > 86400 * 120) {
    drupal_set_message(t('This capture has passed its 120 day limit for issuing refunds.'), 'error');
    drupal_goto('admin/commerce/orders/' . $form_state['order']->order_id . '/payment');
  }
}

/**
 * Confirm form submit callback to refund an order.
 */
function commerce_square_refund_form_submit($form, $form_state) {
  $transaction = $form_state['transaction'];
  $amount = commerce_currency_decimal_to_amount($form_state['values']['amount'], $transaction->currency_code);
  $client = SquareApi::createFromInstanceId($transaction->instance_id);

  try {
    $result = $client->refund($transaction, $amount);
    $refund_transaction = commerce_payment_transaction_new('commerce_square', $transaction->order_id);
    $refund_transaction->instance_id = $transaction->instance_id;
    $refund_transaction->amount = -$amount;
    $refund_transaction->currency_code = $transaction->currency_code;
    $refund_transaction->status = COMMERCE_PAYMENT_STATUS_SUCCESS;
    $remote_transaction = $result->getRefund();
    $tender_id = $remote_transaction->getTenderId();
    $refund_transaction->remote_id = $remote_transaction->getId() . '|' . $tender_id;
    $refund_transaction->order_id = $transaction->order_id;
    $refund_transaction->message = t('Manual refund');
    commerce_payment_transaction_save($refund_transaction);

    drupal_set_message(t('Amount successfully refunded'), 'status');
  }
  catch (ApiException $e) {
    /** @var \SquareConnect\Model\VoidTransactionResponse $result */
    $result = $e->getResponseObject();
    foreach ($result->getErrors() as $error) {
      drupal_set_message(sprintf('%s %s %s', $error->getCode(), $error->getCategory(), $error->getDetail()), 'error');
    }
  }
  catch (Exception $e) {
    drupal_set_message(t('There was an error refunding the transaction: @message', array('@message' => $e->getMessage())), 'error');
  }

  // Redirect to payments page.
  drupal_goto('admin/commerce/orders/' . $transaction->order_id . '/payment');
}
